unit ReaderUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, ExtCtrls, TeeProcs, TeEngine, Chart, RzLstBox, RzChkLst,
  Series, Buttons, Menus;

type
  TfrmZoneBdgtReader = class(TForm)
    OpenDialog1: TOpenDialog;
    chartZONEBDGT: TChart;
    pnlBottom: TPanel;
    SaveDialog1: TSaveDialog;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    About2: TMenuItem;
    Read1: TMenuItem;
    Save1: TMenuItem;
    Exit1: TMenuItem;
    Splitter1: TSplitter;
    Panel1: TPanel;
    cbDecay: TCheckBox;
    cbStoredMass: TCheckBox;
    cbInOut: TCheckBox;
    cbDiscrepancy: TCheckBox;
    rgDataSource: TRadioGroup;
    btnOpen: TSpeedButton;
    btnSave: TSpeedButton;
    BitBtnClose: TBitBtn;
    Panel5: TPanel;
    Panel2: TPanel;
    lblTime: TLabel;
    lblZone: TLabel;
    comboTimeStep: TComboBox;
    comboZone: TComboBox;
    rgPlotType: TRadioGroup;
    Panel4: TPanel;
    Panel6: TPanel;
    Panel7: TPanel;
    cbIn: TCheckBox;
    rzckIn: TRzCheckList;
    Panel8: TPanel;
    Panel3: TPanel;
    cbOut: TCheckBox;
    rzckOut: TRzCheckList;
    Splitter2: TSplitter;
    sbImage: TSpeedButton;
    sbFormat: TSpeedButton;
    Saveasimage1: TMenuItem;
    FormatChart1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnReadClick(Sender: TObject);
    procedure rgPlotTypeClick(Sender: TObject);
    procedure comboZoneChange(Sender: TObject);
    procedure comboTimeStepChange(Sender: TObject);
    procedure rzckChange(Sender: TObject; Index: Integer;
      NewState: TCheckBoxState);
    procedure cbInOutClick(Sender: TObject);
    procedure cbDiscrepancyClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure cbOutClick(Sender: TObject);
    procedure cbInClick(Sender: TObject);
    procedure rgDataSourceClick(Sender: TObject);
    procedure cbDecayClick(Sender: TObject);
    procedure cbStoredMassClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure sbImageClick(Sender: TObject);
    procedure sbFormatClick(Sender: TObject);
    procedure ToolBar1MouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
  private
    function GetNextLine(SearchTerm: string; Start: integer): integer;
    function GetStringBetween(AString, Before, After: String): string;
    function GetColor(AColor: TColor): TColor;
    procedure ReadZonebdgtFile;
    procedure ReadModflowFile;
    procedure ReadMOC3DFile;
    procedure ReadSutraFile;
    procedure ReadMT3DFile;
    function GetNCharacters(var AString: string; N: integer): string;
    procedure ReadFile;
    { Private declarations }
  public
    ZBLStringList : TStringList;
    BudgetList : TList;
    Procedure BudgetListClear;
    procedure PlotResults;
    { Public declarations }
  end;

  TBudget = class(TObject)
    Zone : string;
    StressPeriod : string;
    TimeStep : string;
    TransportStep : string;
    Time : string;
    InList : TList;
    OutList : TList;
    InMinusOut : string;
    Discrepancy : string;
    StoredMass : string;
    Decay : string;
    Constructor Create;
    Destructor Destroy; override;
    function GetStressPeriod : integer;
    function GetTimeStep : integer;
    function GetInMinusOut : double;
    function GetDiscrepancy : double;
    function GetDecay : double;
    function GetStoredMass : double;
    Procedure Clear;
  end;

  TBudgetItem = class(TObject)
    Name : string;
    value : string;
    function RealValue : double;
  end;

var
  frmZoneBdgtReader: TfrmZoneBdgtReader;

implementation

uses AboutUnit, frmFormatUnit;

{$R *.DFM}

var
  DecimalSign : Char;

function InternationalStrToFloat(Value : string) : extended;
var
  DecimalLocation : integer;
//CommaSeparator : array[0..255] of Char;
begin

  if DecimalSign = '.' then
  begin
    DecimalLocation := Pos(',',Value);
    if DecimalLocation > 0 then
    begin
      Value[DecimalLocation] := '.';
    end;
  end;
  if DecimalSign = ',' then
  begin
    DecimalLocation := Pos('.',Value);
    if DecimalLocation > 0 then
    begin
      Value[DecimalLocation] := ',';
    end;
  end;

{  DecimalPosition := Pos(',',Value);
  if DecimalPosition > 0 then
  begin
    Value[DecimalPosition] := '.';
  end; }
  Result := StrToFloat(Value);
end;

procedure TfrmZoneBdgtReader.FormCreate(Sender: TObject);
begin
  ZBLStringList := TStringList.Create;
  BudgetList := TList.Create;
  Constraints.MinHeight := Height-Round(chartZONEBDGT.Height/2);
  Width := rgPlotType.Left + rgPlotType.Width + 8
    + BitBtnClose.Left + BitBtnClose.Width + 12;
  Constraints.MinWidth := Width;
  Panel8.Width := (Panel8.Width + Panel6.Width) div 2;
  Panel1.Constraints.MinHeight := Panel1.Height;
end;

procedure TfrmZoneBdgtReader.FormDestroy(Sender: TObject);
var
  Index : integer;
begin
  ZBLStringList.Free;
  for Index := BudgetList.Count -1 downto 0 do
  begin
    TBudget(BudgetList[Index]).Free;
  end;
  BudgetList.Free;
end;

function TfrmZoneBdgtReader.GetNextLine(SearchTerm : string; Start : integer) : integer;
var
  Index : integer;
begin
  result := -1;
  for Index := Start to ZBLStringList.Count -1 do
  begin
    if Pos(SearchTerm, ZBLStringList.Strings[Index])> 0 then
    begin
      result := Index;
      break;
    end;
  end;
end;

function TfrmZoneBdgtReader.GetNCharacters(var AString : string; N : integer) : string;
begin
  result := Copy(AString, 1, N);
  AString := Copy(AString, N+1, Length(AString));
end;

function TfrmZoneBdgtReader.GetStringBetween(AString, Before, After : String) : string;
begin
  result := AString;
  if Before <> '' then
  begin
    result := Copy(AString,
        Pos(Before,AString)+Length(Before),
        Length(AString));
  end;

  if After <> '' then
  begin
    result := Copy(result,1,Pos(After,result)-1);
  end;

  result := Trim(result);
end;

procedure TfrmZoneBdgtReader.ReadMT3DFile;
var
  SearchTerm : string;
  LineIndex : integer;
  CurrentLine : string;
  TimeStepString, StressPeriodString, TransportStepString, TimeString : string;
  StartLine, StopLine, InnerLineIndex : integer;
  BudgetItem, Rate : string;
  InMinusOut, Discrepancy : string;
  ABudget, Budget1, Budget2 : TBudget;
  ABudgetItem, AnotherBudgetItem, PreviousBudgetItem : TBudgetItem;
  Limit : integer;
  Index, BudgetItemIndex : integer;
  digits : set of Char;
begin
  digits := ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '.'];
  LineIndex := 0;
  SearchTerm := 'TOTAL ELAPSED TIME SINCE';
  LineIndex := GetNextLine(SearchTerm,LineIndex);
  While LineIndex > -1 do
  begin
    ABudget := TBudget.Create;
    BudgetList.Add(ABudget);

    CurrentLine := ZBLStringList.Strings[LineIndex];
    TimeString := GetStringBetween(CurrentLine,'=','');
    while (Length(TimeString) > 0) and
      not (TimeString[Length(TimeString)] in digits) do
    begin
      TimeString := Trim(Copy(TimeString, 1, Length(TimeString)-1));
    end;


    SearchTerm := 'CUMMULATIVE MASS BUDGETS';
    LineIndex := GetNextLine(SearchTerm,LineIndex);
    CurrentLine := ZBLStringList.Strings[LineIndex];
    TransportStepString := GetStringBetween(CurrentLine,'TRANSPORT STEP',',');

    TimeStepString := GetStringBetween(CurrentLine,'TIME STEP',',');

    StressPeriodString := GetStringBetween(CurrentLine,'STRESS PERIOD','');

    ABudget.Zone := 'MT3D Cumulative';
    ABudget.StressPeriod := StressPeriodString;
    ABudget.TimeStep := TimeStepString;
    ABudget.TransportStep := TransportStepString;
    ABudget.Time := TimeString;

    SearchTerm := 'IN';
    LineIndex := GetNextLine(SearchTerm,LineIndex)+2;

    StartLine := LineIndex ;
    SearchTerm := 'TOTAL';
    StopLine :=  GetNextLine(SearchTerm,StartLine+1);

    if StopLine > -1 then
    begin
      for InnerLineIndex := StartLine to StopLine do
      begin
        CurrentLine := ZBLStringList.Strings[InnerLineIndex];
        if Pos(':',CurrentLine) > 0 then
        begin

          BudgetItem := GetStringBetween(CurrentLine, '', ':');
          CurrentLine := Copy(CurrentLine, Pos(':',CurrentLine)+1, Length(CurrentLine));
          Rate := Trim(GetNCharacters(CurrentLine,16));
          ABudgetItem := TBudgetItem.Create;
          ABudgetItem.Name := BudgetItem;
          ABudgetItem.value := Rate;
          ABudget.InList.Add(ABudgetItem);

          if Pos('TOTAL',BudgetItem) > 0 then
          begin
            CurrentLine := Copy(CurrentLine,4,Length(CurrentLine)-7);
          end;
          ABudgetItem := TBudgetItem.Create;
          ABudgetItem.Name := BudgetItem;
          ABudgetItem.value := Trim(CurrentLine);
          if not (ABudgetItem.value[1] in digits) then
          begin
            ABudgetItem.value := Trim(Copy(ABudgetItem.value, 2, Length(ABudgetItem.value)));
          end;
          ABudget.OutList.Add(ABudgetItem);
        end;
      end;
    end;

    if StopLine > -1 then
    begin
      SearchTerm := 'NET (IN - OUT)';
      LineIndex := GetNextLine(SearchTerm,StopLine+1);
      if LineIndex > -1 then
      begin
        CurrentLine := ZBLStringList.Strings[LineIndex];
        InMinusOut := GetStringBetween(CurrentLine,':','');
        ABudget.InMinusOut := InMinusOut;
      end;
    end;

    if LineIndex > -1 then
    begin
      SearchTerm := 'DISCREPANCY';
      LineIndex := GetNextLine(SearchTerm,LineIndex);
      if LineIndex > -1 then
      begin
        CurrentLine := ZBLStringList.Strings[LineIndex];
        Discrepancy := GetStringBetween(CurrentLine,':','');
        ABudget.Discrepancy := Discrepancy;
      end;
    end;

    SearchTerm := 'TOTAL ELAPSED TIME';
    LineIndex := GetNextLine(SearchTerm,LineIndex+1);
  end;
  Limit := BudgetList.Count -1;
  Budget1 := nil;
  for Index := 0 to Limit do
  begin
    Budget2 := BudgetList[Index];
    ABudget := TBudget.Create;
    BudgetList.Add(ABudget);

    ABudget.Zone := 'MT3D Incremental';
    ABudget.StressPeriod := Budget2.StressPeriod;
    ABudget.TimeStep := Budget2.TimeStep;
    ABudget.TransportStep := Budget2.TransportStep;
    ABudget.Time := Budget2.Time;

    for BudgetItemIndex := 0 to Budget2.InList.Count -1 do
    begin
      AnotherBudgetItem := Budget2.InList[BudgetItemIndex];
      ABudgetItem := TBudgetItem.Create;
      ABudgetItem.Name := AnotherBudgetItem.Name;
      if Budget1 = nil then
      begin
        ABudgetItem.value := AnotherBudgetItem.value;
      end
      else
      begin
        PreviousBudgetItem := Budget1.InList[BudgetItemIndex];
        ABudgetItem.value := FloatToStr(StrToFloat(AnotherBudgetItem.value)
          - StrToFloat(PreviousBudgetItem.value));
      end;
      ABudget.InList.Add(ABudgetItem);
    end;

    for BudgetItemIndex := 0 to Budget2.OutList.Count -1 do
    begin
      AnotherBudgetItem := Budget2.OutList[BudgetItemIndex];
      ABudgetItem := TBudgetItem.Create;
      ABudgetItem.Name := AnotherBudgetItem.Name;
      if Budget1 = nil then
      begin
        ABudgetItem.value := AnotherBudgetItem.value;
      end
      else
      begin
        PreviousBudgetItem := Budget1.OutList[BudgetItemIndex];
        ABudgetItem.value := FloatToStr(StrToFloat(AnotherBudgetItem.value)
          - StrToFloat(PreviousBudgetItem.value));
      end;
      ABudget.OutList.Add(ABudgetItem);
    end;

    if Budget1 = nil then
    begin
      ABudget.InMinusOut := Budget2.InMinusOut;
      ABudget.Discrepancy := Budget2.Discrepancy;
    end
    else
    begin
      ABudget.InMinusOut := FloatToStr(StrToFloat(Budget2.InMinusOut)
          - StrToFloat(Budget1.InMinusOut));
      ABudget.Discrepancy := FloatToStr(StrToFloat(Budget2.Discrepancy)
          - StrToFloat(Budget1.Discrepancy));
    end;



    Budget1 := Budget2;
  end;

end;

procedure TfrmZoneBdgtReader.ReadMOC3DFile;
var
  SearchTerm : string;
  LineIndex : integer;
  CurrentLine : string;
//  CurrentZone : String;
  TimeStepString, StressPeriodString, TransportStepString, TimeString : string;
  StoredMass : string;
  StartLine, StopLine, InnerLineIndex : integer;
  BudgetItem, Rate : string;
  InMinusOut, Discrepancy, Decay : string;
  ABudget : TBudget;
  ABudgetItem : TBudgetItem;
begin
    LineIndex := 0;
    SearchTerm := 'SOLUTE BUDGET';
    LineIndex := GetNextLine(SearchTerm,LineIndex);
    While LineIndex > -1 do
    begin
      ABudget := TBudget.Create;
      BudgetList.Add(ABudget);

      SearchTerm := 'STRESS PERIOD';
      LineIndex := GetNextLine(SearchTerm,LineIndex);
      CurrentLine := ZBLStringList.Strings[LineIndex];
      StressPeriodString := GetStringBetween(CurrentLine,SearchTerm,'OUT OF');

      SearchTerm := 'FLOW TIME STEP';
      LineIndex := GetNextLine(SearchTerm,LineIndex);
      CurrentLine := ZBLStringList.Strings[LineIndex];
      TimeStepString := GetStringBetween(CurrentLine,SearchTerm,'OUT OF');

      SearchTerm := 'TRANSPORT TIME INCREMENT';
      LineIndex := GetNextLine(SearchTerm,LineIndex);
      CurrentLine := ZBLStringList.Strings[LineIndex];
      TransportStepString := GetStringBetween(CurrentLine,SearchTerm,'OUT OF');

      SearchTerm := 'ELAPSED TIME';
      LineIndex := GetNextLine(SearchTerm,LineIndex);
      CurrentLine := ZBLStringList.Strings[LineIndex];
      TimeString := GetStringBetween(CurrentLine,'=','');

      SearchTerm := 'CHANGE IN MASS STORED';
      LineIndex := GetNextLine(SearchTerm,LineIndex);
      CurrentLine := ZBLStringList.Strings[LineIndex];
      StoredMass := GetStringBetween(CurrentLine,'=','');

      ABudget.Zone := 'MOC3D';
      ABudget.StressPeriod := StressPeriodString;
      ABudget.TimeStep := TimeStepString;
      ABudget.TransportStep := TransportStepString;
      ABudget.Time := TimeString;
      ABudget.StoredMass := StoredMass;

      SearchTerm := 'CUMULATIVE SOLUTE MASS';
      LineIndex := GetNextLine(SearchTerm,LineIndex)+1;
      SearchTerm := 'IN';
      LineIndex := GetNextLine(SearchTerm,LineIndex)+1;

      StartLine := LineIndex + 1;
      SearchTerm := 'OUT:';
      StopLine :=  GetNextLine(SearchTerm,StartLine+1);

      if StopLine > -1 then
      begin
        for InnerLineIndex := StartLine to StopLine do
        begin
          CurrentLine := ZBLStringList.Strings[InnerLineIndex];
          if Pos('=',CurrentLine) > 0 then
          begin

            BudgetItem := GetStringBetween(CurrentLine,'','=');
            Rate := GetStringBetween(CurrentLine,'=','');
            ABudgetItem := TBudgetItem.Create;
            ABudgetItem.Name := BudgetItem;
            ABudgetItem.value := Rate;
            ABudget.InList.Add(ABudgetItem);
          end;
        end;
      end;
      if StopLine > -1 then
      begin
        StartLine := StopLine + 1;
        SearchTerm := 'SOURCE-TERM DECAY';
        StopLine :=  GetNextLine(SearchTerm,StartLine+1)-1;
        if StopLine > -1 then
        begin
          for InnerLineIndex := StartLine to StopLine do
          begin
            CurrentLine := ZBLStringList.Strings[InnerLineIndex];
            if Pos('=',CurrentLine) > 0 then
            begin
              BudgetItem := GetStringBetween(CurrentLine,'','=');
              Rate := GetStringBetween(CurrentLine,'=','');
              ABudgetItem := TBudgetItem.Create;
              ABudgetItem.Name := BudgetItem;
              ABudgetItem.value := Rate;
              ABudget.OutList.Add(ABudgetItem);
            end;
          end;
        end;
      end;

      if StopLine > -1 then
      begin
        SearchTerm := 'SOURCE-TERM DECAY';
        LineIndex := GetNextLine(SearchTerm,StopLine+1);
        if LineIndex > -1 then
        begin
          CurrentLine := ZBLStringList.Strings[LineIndex];
          Decay := GetStringBetween(CurrentLine,'=','');
          ABudget.Decay := Decay;
        end;
      end;

      if LineIndex > -1 then
      begin
        SearchTerm := 'RESIDUAL';
        LineIndex := GetNextLine(SearchTerm,LineIndex);
        if LineIndex > -1 then
        begin
          CurrentLine := ZBLStringList.Strings[LineIndex];
          InMinusOut := GetStringBetween(CurrentLine,'=','');
          ABudget.InMinusOut := InMinusOut;
        end;
      end;

      if LineIndex > -1 then
      begin
        SearchTerm := 'PERCENT DISCREPANCY';
        LineIndex := GetNextLine(SearchTerm,LineIndex);
        if LineIndex > -1 then
        begin
          CurrentLine := ZBLStringList.Strings[LineIndex];
          SearchTerm := 'RELATIVE';
          if Pos(SearchTerm,CurrentLine)>0 then
          begin
            Discrepancy := GetStringBetween(CurrentLine,'=',SearchTerm);
          end
          else
          begin
            Discrepancy := GetStringBetween(CurrentLine,'=','');
          end;
          ABudget.Discrepancy := Discrepancy;
        end;
      end;
      SearchTerm := 'SOLUTE BUDGET';
      LineIndex := GetNextLine(SearchTerm,LineIndex+1);
    end;
end;


procedure TfrmZoneBdgtReader.ReadZonebdgtFile;
var
  SearchTerm : string;
  LineIndex : integer;
  CurrentLine : string;
  CurrentZone : String;
  TimeStepString, StressPeriodString : string;
  StartLine, StopLine, InnerLineIndex : integer;
  BudgetItem, Rate : string;
  InMinusOut, Discrepancy : string;
  ABudget : TBudget;
  ABudgetItem : TBudgetItem;
begin
    LineIndex := 0;
    SearchTerm := 'Flow Budget for ';
    LineIndex := GetNextLine(SearchTerm,LineIndex);
    While LineIndex > -1 do
    begin
      ABudget := TBudget.Create;
      BudgetList.Add(ABudget);
      CurrentLine := ZBLStringList.Strings[LineIndex];

      CurrentZone := GetStringBetween(CurrentLine,SearchTerm,'at');
      CurrentLine := GetStringBetween(CurrentLine,CurrentZone,'');

      TimeStepString := GetStringBetween(CurrentLine,'Time Step','of');
      CurrentLine := GetStringBetween(CurrentLine,TimeStepString,'');

      StressPeriodString := GetStringBetween(CurrentLine,'Stress Period','');

      ABudget.Zone := CurrentZone;
      ABudget.StressPeriod := StressPeriodString;
      ABudget.TimeStep := TimeStepString;

      StartLine := LineIndex + 1;
      SearchTerm := 'OUT:';
      StopLine :=  GetNextLine(SearchTerm,StartLine+1);

      if StopLine > -1 then
      begin
        for InnerLineIndex := StartLine to StopLine do
        begin
          CurrentLine := ZBLStringList.Strings[InnerLineIndex];
          if Pos('=',CurrentLine) > 0 then
          begin

            BudgetItem := GetStringBetween(CurrentLine,'','=');
            Rate := GetStringBetween(CurrentLine,'=','');
            ABudgetItem := TBudgetItem.Create;
            ABudgetItem.Name := BudgetItem;
            ABudgetItem.value := Rate;
            ABudget.InList.Add(ABudgetItem);
          end;
        end;
      end;
      if StopLine > -1 then
      begin
        StartLine := StopLine + 1;
        SearchTerm := 'IN - OUT';
        StopLine :=  GetNextLine(SearchTerm,StartLine+1)-1;
        if StopLine > -1 then
        begin
          for InnerLineIndex := StartLine to StopLine do
          begin
            CurrentLine := ZBLStringList.Strings[InnerLineIndex];
            if Pos('=',CurrentLine) > 0 then
            begin
              BudgetItem := GetStringBetween(CurrentLine,'','=');
              Rate := GetStringBetween(CurrentLine,'=','');
              ABudgetItem := TBudgetItem.Create;
              ABudgetItem.Name := BudgetItem;
              ABudgetItem.value := Rate;
              ABudget.OutList.Add(ABudgetItem);
            end;
          end;
        end;
      end;

      if StopLine > -1 then
      begin
        SearchTerm := 'IN - OUT';
        LineIndex := GetNextLine(SearchTerm,StopLine+1);
        if LineIndex > -1 then
        begin
          CurrentLine := ZBLStringList.Strings[LineIndex];
          InMinusOut := GetStringBetween(CurrentLine,'=','');
          ABudget.InMinusOut := InMinusOut;
        end;
      end;


      if LineIndex > -1 then
      begin
        SearchTerm := 'Percent Discrepancy';
        LineIndex := GetNextLine(SearchTerm,StopLine);
        if LineIndex > -1 then
        begin
          CurrentLine := ZBLStringList.Strings[LineIndex];
          Discrepancy := GetStringBetween(CurrentLine,'=','');
          ABudget.Discrepancy := Discrepancy;
        end;
      end;


      if LineIndex > -1 then
      begin
        SearchTerm := 'Flow Budget for ';
        LineIndex := GetNextLine(SearchTerm,LineIndex+1);
      end;
    end;
end;

procedure TfrmZoneBdgtReader.ReadSutraFile;
var
  SearchTerm : string;
  LineIndex : integer;
  CurrentLine : string;
  TimeStepString : string;
  StartLine, StopLine, InnerLineIndex : integer;
  BudgetItem, Rate : string;
  ABudget : TBudget;
  ABudgetItem : TBudgetItem;
  ReplacementsNames : TStringList;
  ReplacementIndex : Integer;
begin
  ReplacementsNames := TStringList.Create;
  try
    begin
      ReplacementsNames.Add('NET RATE OF INCREASE(+)/DECREASE(-) OF SOLUTE DUE TO CONCENTRATION CHANGE');
      ReplacementsNames.Add('Solute Change due to Concentration Change');

      ReplacementsNames.Add('NET RATE OF INCREASE(+)/DECREASE(-) OF ADSORBATE');
      ReplacementsNames.Add('Adsorbate Change');

      ReplacementsNames.Add('NET RATE OF INCREASE(+)/DECREASE(-) OF SOLUTE DUE TO CHANGE IN MASS OF FLUID');
      ReplacementsNames.Add('Solute Change due to Change in Fluid Mass');

      ReplacementsNames.Add('NET FIRST-ORDER PRODUCTION(+)/DECAY(-) OF SOLUTE');
      ReplacementsNames.Add('1''st-Order Production of Solute');

      ReplacementsNames.Add('NET FIRST-ORDER PRODUCTION(+)/DECAY(-) OF ADSORBATE');
      ReplacementsNames.Add('1''st-Order Production of Adsorbate');

      ReplacementsNames.Add('NET ZERO-ORDER PRODUCTION(+)/DECAY(-) OF SOLUTE');
      ReplacementsNames.Add('Zero-Order Production of Solute');

      ReplacementsNames.Add('NET ZERO-ORDER PRODUCTION(+)/DECAY(-) OF ADSORBATE');
      ReplacementsNames.Add('Zero-Order Production of Adsorbate');

      ReplacementsNames.Add('NET GAIN(+)/LOSS(-) OF SOLUTE THROUGH FLUID SOURCES AND SINKS');
      ReplacementsNames.Add('Gain of Solute through Fluid Sources and Sinks');

      ReplacementsNames.Add('NET GAIN(+)/LOSS(-) OF SOLUTE THROUGH INFLOWS OR OUTFLOWS AT POINTS OF SPECIFIED PRESSURE');
      ReplacementsNames.Add('Gain of Solute through Flows at Specified Pressure Points');

      ReplacementsNames.Add('NET GAIN(+)/LOSS(-) OF SOLUTE THROUGH SOLUTE SOURCES AND SINKS');
      ReplacementsNames.Add('Gain of Solute through Solute Sources');

      ReplacementsNames.Add('NET GAIN(+)/LOSS(-)'' OF SOLUTE AT POINTS OF SPECIFIED CONCENTRATION');
      ReplacementsNames.Add('Gain of Solute at Specified Concentration Points');

      ReplacementsNames.Add('NET GAIN(+)/LOSS(-) OF SOLUTE AT POINTS OF SPECIFIED CONCENTRATION');
      ReplacementsNames.Add('Gain of Solute at Specified Concentration Points');

      ReplacementsNames.Add('RATE OF CHANGE IN TOTAL STORED FLUID DUE TO PRESSURE CHANGE, INCREASE(+)/DECREASE(-)');
      ReplacementsNames.Add('Change in Fluid due to Pressure Change');

      ReplacementsNames.Add('RATE OF CHANGE IN TOTAL STORED FLUID DUE TO CONCENTRATION CHANGE, INCREASE(+)/DECREASE(-)');
      ReplacementsNames.Add('Change in Fluid due to Concentration Change');

      ReplacementsNames.Add('TOTAL OF FLUID SOURCES AND SINKS, NET INFLOW(+)/NET OUTFLOW(-)');
      ReplacementsNames.Add('Total of Fluid Sources and Sinks');

      ReplacementsNames.Add('TOTAL OF FLUID FLOWS AT POINTS OF SPECIFIED PRESSURE, NET INFLOW(+)/NET OUTFLOW(-)');
      ReplacementsNames.Add('Total of Fluid Flows at Points of Specified Pressure');

      ReplacementsNames.Add('RATE OF CHANGE IN TOTAL STORED FLUID DUE TO  TEMPERATURE  CHANGE, INCREASE(+)/DECREASE(-)');
      ReplacementsNames.Add('Change in Fluid due to Temperature Change');

      ReplacementsNames.Add('NET RATE OF INCREASE(+)/DECREASE(-) OF ENERGY IN FLUID DUE TO TEMPERATURE CHANGE');
      ReplacementsNames.Add('Energy Change in Fluid due to Temperature Change');

      ReplacementsNames.Add('NET RATE OF INCREASE(+)/DECREASE(-) OF ENERGY IN SOLID GRAINS');
      ReplacementsNames.Add('Energy Change in Solid Grains');

      ReplacementsNames.Add('NET RATE OF INCREASE(+)/DECREASE(-) OF ENERGY IN FLUID DUE TO CHANGE IN MASS OF FLUID');
      ReplacementsNames.Add('Energy Change in Fluid due to Fluid Mass Change');

      ReplacementsNames.Add('NET ZERO-ORDER PRODUCTION(+)/LOSS(-) OF ENERGY IN FLUID');
      ReplacementsNames.Add('Zero-Order Production/Loss of Energy in Fluid');

      ReplacementsNames.Add('NET ZERO-ORDER PRODUCTION(+)/LOSS(-) OF ENERGY IN SOLID GRAINS');
      ReplacementsNames.Add('Zero-Order Production/Loss of Energy in Solid Grains');

      ReplacementsNames.Add('NET GAIN(+)/LOSS(-) OF ENERGY THROUGH FLUID SOURCES AND SINKS');
      ReplacementsNames.Add('Energy Gain/Loss through Fluid Sources and Sinks');

      ReplacementsNames.Add('NET GAIN(+)/LOSS(-) OF ENERGY THROUGH INFLOWS OR OUTFLOWS AT POINTS OF SPECIFIED PRESSURE');
      ReplacementsNames.Add('Energy Gain/Loss through flows at Specified Pressure Points');

      ReplacementsNames.Add('NET GAIN(+)/LOSS(-) OF ENERGY THROUGH ENERGY SOURCES AND SINKS');
      ReplacementsNames.Add('Energy Gain/Loss through Energy Sources and Sinks');

      ReplacementsNames.Add('NET GAIN(+)''/LOSS(-) OF ENERGY THROUGH POINTS OF SPECIFIED TEMPERATURE');
      ReplacementsNames.Add('Energy Gain/Loss through Points of Specified Temperature');

      ReplacementsNames.Add('NET GAIN(+)/LOSS(-) OF ENERGY THROUGH POINTS OF SPECIFIED TEMPERATURE');
      ReplacementsNames.Add('Energy Gain/Loss through Points of Specified Temperature');

      LineIndex := 0;
      SearchTerm := 'S O L U T E   B U D G E T';
      LineIndex := GetNextLine(SearchTerm,LineIndex);
      While LineIndex > -1 do
      begin
        ABudget := TBudget.Create;
        BudgetList.Add(ABudget);
        CurrentLine := ZBLStringList.Strings[LineIndex];

        TimeStepString := GetStringBetween(CurrentLine,'TIME STEP',',');

        ABudget.Zone := 'SUTRA Solute Budget';
        ABudget.TimeStep := TimeStepString;

        StartLine := LineIndex + 1;
        SearchTerm := 'SOLUTE SOURCES OR SINKS';
        StopLine :=  GetNextLine(SearchTerm,StartLine+1)-1;

        if StopLine > -1 then
        begin
          for InnerLineIndex := StartLine to StopLine do
          begin
            CurrentLine := ZBLStringList.Strings[InnerLineIndex];
            if Pos('NET',CurrentLine) > 0 then
            begin

              Rate := GetStringBetween(CurrentLine,'','NET');
              BudgetItem := GetStringBetween(CurrentLine,Rate,'');
              ABudgetItem := TBudgetItem.Create;
              ReplacementIndex := ReplacementsNames.IndexOf(BudgetItem);
              if  ReplacementIndex > -1 then
              begin
                BudgetItem := ReplacementsNames[ReplacementIndex +1];
              end;
              ABudgetItem.Name := BudgetItem;
              ABudgetItem.value := Rate;
              ABudget.InList.Add(ABudgetItem);
            end;
          end;
        end;
        SearchTerm := 'S O L U T E   B U D G E T';
        LineIndex := GetNextLine(SearchTerm,LineIndex+1);
      end;

      LineIndex := 0;
      SearchTerm := 'F L U I D   M A S S   B U D G E T';
      LineIndex := GetNextLine(SearchTerm,LineIndex);
      While LineIndex > -1 do
      begin
        ABudget := TBudget.Create;
        BudgetList.Add(ABudget);
        CurrentLine := ZBLStringList.Strings[LineIndex];

        TimeStepString := GetStringBetween(CurrentLine,'TIME STEP',',');

        ABudget.Zone := 'SUTRA Mass Budget';
        ABudget.TimeStep := TimeStepString;

        StartLine := LineIndex + 1;
        SearchTerm := 'FLUID SOURCES OR SINKS';
        StopLine :=  GetNextLine(SearchTerm,StartLine+1)-1;

        if StopLine > -1 then
        begin
          for InnerLineIndex := StartLine to StopLine do
          begin
            CurrentLine := ZBLStringList.Strings[InnerLineIndex];
            if Pos('RATE OF',CurrentLine) > 0 then
            begin

              Rate := GetStringBetween(CurrentLine,'','RATE OF');
              BudgetItem := GetStringBetween(CurrentLine,Rate,'');
              ABudgetItem := TBudgetItem.Create;
              ReplacementIndex := ReplacementsNames.IndexOf(BudgetItem);
              if  ReplacementIndex > -1 then
              begin
                BudgetItem := ReplacementsNames[ReplacementIndex +1];
              end;
              ABudgetItem.Name := BudgetItem;
              ABudgetItem.value := Rate;
              ABudget.InList.Add(ABudgetItem);
            end;
            if Pos('TOTAL OF',CurrentLine) > 0 then
            begin

              Rate := GetStringBetween(CurrentLine,'','TOTAL OF');
              BudgetItem := GetStringBetween(CurrentLine,Rate,'');
              ABudgetItem := TBudgetItem.Create;
              ReplacementIndex := ReplacementsNames.IndexOf(BudgetItem);
              if  ReplacementIndex > -1 then
              begin
                BudgetItem := ReplacementsNames[ReplacementIndex +1];
              end;
              ABudgetItem.Name := BudgetItem;
              ABudgetItem.value := Rate;
              ABudget.InList.Add(ABudgetItem);
            end;
          end;
        end;
        SearchTerm := 'F L U I D   M A S S   B U D G E T';
        LineIndex := GetNextLine(SearchTerm,LineIndex+1);
      end;





      LineIndex := 0;
      SearchTerm := 'E N E R G Y   B U D G E T';
      LineIndex := GetNextLine(SearchTerm,LineIndex);
      While LineIndex > -1 do
      begin
        ABudget := TBudget.Create;
        BudgetList.Add(ABudget);
        CurrentLine := ZBLStringList.Strings[LineIndex];

        TimeStepString := GetStringBetween(CurrentLine,'TIME STEP',',');

        ABudget.Zone := 'SUTRA Energy Budget';
        ABudget.TimeStep := TimeStepString;

        StartLine := LineIndex + 1;
        SearchTerm := 'ENERGY SOURCES OR SINKS';
        StopLine :=  GetNextLine(SearchTerm,StartLine+1)-1;

        if StopLine > -1 then
        begin
          for InnerLineIndex := StartLine to StopLine do
          begin
            CurrentLine := ZBLStringList.Strings[InnerLineIndex];
            if Pos('NET',CurrentLine) > 0 then
            begin

              Rate := GetStringBetween(CurrentLine,'','NET');
              BudgetItem := GetStringBetween(CurrentLine,Rate,'');
              ABudgetItem := TBudgetItem.Create;
              ReplacementIndex := ReplacementsNames.IndexOf(BudgetItem);
              if  ReplacementIndex > -1 then
              begin
                BudgetItem := ReplacementsNames[ReplacementIndex +1];
              end;
              ABudgetItem.Name := BudgetItem;
              ABudgetItem.value := Rate;
              ABudget.InList.Add(ABudgetItem);
            end;
          end;
        end;
        SearchTerm := 'E N E R G Y   B U D G E T';
        LineIndex := GetNextLine(SearchTerm,LineIndex+1);
      end;


    end;
  finally
    begin
      ReplacementsNames.Free;
    end;
  end;
end;

procedure TfrmZoneBdgtReader.ReadModflowFile;
var
  SearchTerm : string;
  LineIndex : integer;
  CurrentLine : string;
  TimeStepString, StressPeriodString : string;
  StartLine, StopLine, InnerLineIndex : integer;
  BudgetItem, Rate : string;
  InMinusOut, Discrepancy : string;
  CumBudget, RateBudget : TBudget;
  CumBudgetItem, RateBudgetItem : TBudgetItem;
  StreamBudget : TBudget;
  Step : Integer;
  FoundEnd : boolean;
  Segment : String;
  Reach : String;
  StreamBudgetItem : TBudgetItem;
  StreamAquiferBudgetItem : TBudget;
  BudgetItemPrefix : string;
begin
  LineIndex := 0;
  SearchTerm := 'VOLUMETRIC BUDGET FOR ENTIRE MODEL';
  LineIndex := GetNextLine(SearchTerm,LineIndex);
  While LineIndex > -1 do
  begin
    CumBudget := TBudget.Create;
    RateBudget := TBudget.Create;
    BudgetList.Add(CumBudget);
    BudgetList.Add(RateBudget);
    CumBudget.Zone := 'CUMULATIVE';
    RateBudget.Zone := 'RATES';

    CurrentLine := ZBLStringList.Strings[LineIndex];

    TimeStepString := GetStringBetween(CurrentLine,'TIME STEP','IN STRESS PERIOD');
    CurrentLine := GetStringBetween(CurrentLine,TimeStepString,'');

    StressPeriodString := GetStringBetween(CurrentLine,'STRESS PERIOD','');

    CumBudget.StressPeriod := StressPeriodString;
    CumBudget.TimeStep := TimeStepString;
    RateBudget.StressPeriod := StressPeriodString;
    RateBudget.TimeStep := TimeStepString;

    StartLine := LineIndex + 1;
    SearchTerm := 'IN:';
    StartLine :=  GetNextLine(SearchTerm,StartLine+1);

    SearchTerm := 'OUT:';
    StopLine :=  GetNextLine(SearchTerm,StartLine+1);

    if StopLine > -1 then
    begin
      for InnerLineIndex := StartLine to StopLine do
      begin
        CurrentLine := ZBLStringList.Strings[InnerLineIndex];
        if Pos('=',CurrentLine) > 0 then
        begin

          BudgetItem := GetStringBetween(CurrentLine,'','=');
          if BudgetItem[1] = '0' then
          begin
            BudgetItem := Trim(Copy(BudgetItem, 2, Length(BudgetItem)));
          end;
          CurrentLine := GetStringBetween(CurrentLine,'=','');

          CumBudgetItem := TBudgetItem.Create;
          RateBudgetItem := TBudgetItem.Create;

          CumBudget.InList.Add(CumBudgetItem);
          RateBudget.InList.Add(RateBudgetItem);

          CumBudgetItem.Name := BudgetItem;
          RateBudgetItem.Name := BudgetItem;

          Rate := GetStringBetween(CurrentLine,'',BudgetItem);
          CumBudgetItem.value := Rate;

          CurrentLine := GetStringBetween(CurrentLine,BudgetItem,'');
          Rate := GetStringBetween(CurrentLine,'=','');
          RateBudgetItem.value := Rate;
        end;
      end;
    end;
    if StopLine > -1 then
    begin
      StartLine := StopLine + 1;
      SearchTerm := 'IN - OUT';
      StopLine :=  GetNextLine(SearchTerm,StartLine+1)-1;
      if StopLine > -1 then
      begin
        for InnerLineIndex := StartLine to StopLine do
        begin
          CurrentLine := ZBLStringList.Strings[InnerLineIndex];
          if Pos('=',CurrentLine) > 0 then
          begin
            BudgetItem := GetStringBetween(CurrentLine,'','=');
            if BudgetItem[1] = '0' then
            begin
              BudgetItem := Trim(Copy(BudgetItem, 2, Length(BudgetItem)));
            end;
            CurrentLine := GetStringBetween(CurrentLine,'=','');

            CumBudgetItem := TBudgetItem.Create;
            RateBudgetItem := TBudgetItem.Create;

            CumBudget.OutList.Add(CumBudgetItem);
            RateBudget.OutList.Add(RateBudgetItem);

            CumBudgetItem.Name := BudgetItem;
            RateBudgetItem.Name := BudgetItem;

            Rate := GetStringBetween(CurrentLine,'',BudgetItem);
            CumBudgetItem.value := Rate;

            CurrentLine := GetStringBetween(CurrentLine,BudgetItem,'');
            Rate := GetStringBetween(CurrentLine,'=','');
            RateBudgetItem.value := Rate;
          end;
        end;
      end;
    end;

    if StopLine > -1 then
    begin
      SearchTerm := 'IN - OUT';
      LineIndex := GetNextLine(SearchTerm,StopLine+1);
      if LineIndex > -1 then
      begin
        CurrentLine := ZBLStringList.Strings[LineIndex];
        CurrentLine := GetStringBetween(CurrentLine,'=','');
        InMinusOut := GetStringBetween(CurrentLine,'',SearchTerm);
        CumBudget.InMinusOut := InMinusOut;

        CurrentLine := GetStringBetween(CurrentLine,SearchTerm,'');
        InMinusOut := GetStringBetween(CurrentLine,'=','');
        RateBudget.InMinusOut := InMinusOut;
      end;
    end;


    if LineIndex > -1 then
    begin
      SearchTerm := 'PERCENT DISCREPANCY';
      LineIndex := GetNextLine(SearchTerm,StopLine);
      if LineIndex > -1 then
      begin
        CurrentLine := ZBLStringList.Strings[LineIndex];
        CurrentLine := GetStringBetween(CurrentLine,'=','');
        Discrepancy := GetStringBetween(CurrentLine,'',SearchTerm);
        CumBudget.Discrepancy := Discrepancy;

        CurrentLine := GetStringBetween(CurrentLine,SearchTerm,'');
        Discrepancy := GetStringBetween(CurrentLine,'=','');
        RateBudget.Discrepancy := Discrepancy;
      end;
    end;

    SearchTerm := 'VOLUMETRIC BUDGET FOR ENTIRE MODEL';
    LineIndex := GetNextLine(SearchTerm,LineIndex+1);
  end;

  Step := 0;
  LineIndex := 0;
  SearchTerm := 'LAYER      ROW     COLUMN     STREAM    REACH      FLOW INTO    FLOW INTO      FLOW OUT OF';
  LineIndex := GetNextLine(SearchTerm,LineIndex);
  While LineIndex > -1 do
  begin
    Inc(Step);

    StreamBudget := TBudget.Create;
    BudgetList.Add(StreamBudget);
    StreamBudget.Zone := 'Stream Flow';
    StreamBudget.InMinusOut := '0';
    StreamBudget.Discrepancy := '0';
    StreamBudget.StressPeriod := '0';
    StreamBudget.TimeStep := IntToStr(Step);

    StreamAquiferBudgetItem := TBudget.Create;
    BudgetList.Add(StreamAquiferBudgetItem);
    StreamAquiferBudgetItem.Zone := 'Stream-Aquifer';
    StreamAquiferBudgetItem.InMinusOut := '0';
    StreamAquiferBudgetItem.Discrepancy := '0';
    StreamAquiferBudgetItem.StressPeriod := '0';
    StreamAquiferBudgetItem.TimeStep := IntToStr(Step);

    StartLine := LineIndex + 4;
    StopLine := StartLine - 1;
    LineIndex := StartLine ;
    FoundEnd := False;
    While not FoundEnd do
    begin
      if (LineIndex > ZBLStringList.Count -1)
        or (ZBLStringList[LineIndex] = '1')
        or (ZBLStringList[LineIndex] = '') then
      begin
        StopLine := LineIndex-1;
        FoundEnd := True;
      end;
      Inc(LineIndex);
    end;

    for InnerLineIndex := StartLine to StopLine do
    begin
      CurrentLine := ZBLStringList.Strings[InnerLineIndex];

      Segment := Trim(Copy(CurrentLine,37,10));
      Reach := Trim(Copy(CurrentLine,47,10));

      BudgetItemPrefix := 'Segment: ' + Segment + '; Reach: ' + Reach + '; ';

      StreamBudgetItem := TBudgetItem.Create;
      StreamBudget.InList.Add(StreamBudgetItem);
      StreamBudgetItem.Name := BudgetItemPrefix + 'FLOW INTO STREAM REACH';
      Rate := Trim(Copy(CurrentLine,65,9));
      StreamBudgetItem.value := Rate;

      StreamBudgetItem := TBudgetItem.Create;
      StreamAquiferBudgetItem.InList.Add(StreamBudgetItem);
      StreamBudgetItem.Name := BudgetItemPrefix + 'FLOW INTO AQUIFER';
      Rate := Trim(Copy(CurrentLine,79,9));
      StreamBudgetItem.value := Rate;

      StreamBudgetItem := TBudgetItem.Create;
      StreamBudget.OutList.Add(StreamBudgetItem);
      StreamBudgetItem.Name := BudgetItemPrefix + 'FLOW OUT OF STREAM REACH';
      Rate := Trim(Copy(CurrentLine,96,9));
      StreamBudgetItem.value := Rate;

    end;

    SearchTerm := 'LAYER      ROW     COLUMN     STREAM    REACH      FLOW INTO    FLOW INTO      FLOW OUT OF';
    LineIndex := GetNextLine(SearchTerm,LineIndex+1);
  end;
end;

procedure TfrmZoneBdgtReader.ReadFile;
var
  F : TextFile;
  S : String;
begin
  AssignFile(F, OpenDialog1.FileName);
  try
    Reset(F);
    While not EOF(F) do
    begin
      Readln(F, S);
      ZBLStringList.Add(S)
    end;
  finally
    CloseFile(F);
  end;
end;


procedure TfrmZoneBdgtReader.btnReadClick(Sender: TObject);
begin

  If OpenDialog1.Execute then
  begin
    frmZoneBdgtReader.Cursor := crHourglass;
    try
      Caption := 'Budgeteer: ' + OpenDialog1.FileName;
      Save1.Enabled := True;
      btnSave.Enabled := True;
      rzckIn.Enabled := True;
      rzckOut.Enabled := True;
      sbImage.Enabled := True;
      sbFormat.Enabled := True;
      Saveasimage1.Enabled := True;
      FormatChart1.Enabled := True;
      rzckIn.Color := clWindow;
      rzckOut.Color := clWindow;
      comboZone.Enabled := True;
      comboZone.Color := clWindow;
      BudgetListClear;
      ZBLStringList.Clear;
      ReadFile;
  //    ZBLStringList.LoadFromFile(OpenDialog1.FileName);
      Case rgDataSource.ItemIndex of
        0:
          begin
            ReadZonebdgtFile;
          end;
        1:
          begin
            ReadModflowFile;
          end;
        2:
          begin
            ReadMOC3DFile;
          end;
        3:
          begin
            ReadSUTRAFile;
          end;
        4:
          begin
            ReadMT3DFile;
          end;
      end;

      rgPlotTypeClick(nil);
    finally
      frmZoneBdgtReader.Cursor := crDefault;
    end;
  end;
end;

{ TBudget }

procedure TBudget.Clear;
var
  Index : Integer;
begin
  for Index := InList.Count -1 downto 0 do
  begin
    TBudgetItem(InList[Index]).Free;
  end;
  for Index := OutList.Count -1 downto 0 do
  begin
    TBudgetItem(OutList[Index]).Free;
  end;
  InList.Clear;
  OutList.Clear;
end;

constructor TBudget.Create;
begin
  InList := TList.Create;
  OutList := TList.Create;
end;

destructor TBudget.Destroy;
begin
  Clear;
  InList.Free;
  OutList.Free;
  inherited;
end;

function TBudget.GetDecay: double;
begin
  try
    result := InternationalStrToFloat(Decay);
  except on EConvertError do
    begin
      result := 0;
    end;
  end;
end;

function TBudget.GetDiscrepancy: double;
begin
  try
    result := InternationalStrToFloat(Discrepancy);
  except on EConvertError do
    begin
      result := 0;
    end;
  end;

end;

function TBudget.GetInMinusOut: double;
begin
  try
    result := InternationalStrToFloat(InMinusOut);
  except on EConvertError do
    begin
      result := 0;
    end;
  end;
end;

function TBudget.GetStoredMass: double;
begin
  try
    result := InternationalStrToFloat(StoredMass);
  except on EConvertError do
    begin
      result := 0;
    end;
  end;
end;

function TBudget.GetStressPeriod: integer;
begin
  try
    result := StrToInt(StressPeriod);
  except on EConvertError do
    begin
      result := 0;
    end;
  end;

end;

function TBudget.GetTimeStep: integer;
begin
  try
    result := StrToInt(TimeStep);
  except on EConvertError do
    begin
      result := 0;
    end;
  end;
end;

{ TBudgetItem }

function TBudgetItem.RealValue: double;
var
  DPosition : integer;
begin
  try
    DPosition := Pos('D',Value);
    if DPosition > 0 then
    begin
      Value[DPosition] := 'E';
    end;
    result := InternationalStrToFloat(Value);
  except on EConvertError do
    begin
      result := 0;
    end;
  end;

end;

procedure TfrmZoneBdgtReader.rgPlotTypeClick(Sender: TObject);
var
  Index : integer;
  ABudget : TBudget;
begin
  comboZone.Items.Clear;
  for Index := 0 to BudgetList.Count - 1 do
  begin
    ABudget := BudgetList[Index];
    if comboZone.Items.IndexOf(ABudget.Zone) = -1 then
    begin
      comboZone.Items.Add(ABudget.Zone);
    end;
  end;
  if comboZone.Items.Count > 0 then
  begin
    comboZone.ItemIndex := 0;
  end;
  comboTimeStep.Enabled := (rgPlotType.ItemIndex = 0);
  if comboTimeStep.Enabled then
  begin
    comboTimeStep.Color := clWindow;
  end
  else
  begin
    comboTimeStep.Color := clBtnFace;
  end;
  comboZoneChange(nil);
  PlotResults;

end;

function TimeText(ABudget : TBudget) : string;
begin
  if Pos('SUTRA',ABudget.Zone) > 0  then
  begin
    result := 'Time Step: ' + ABudget.TimeStep;
  end
  else
  begin
    result := 'Stress Period: ' + ABudget.StressPeriod
            + '; Time Step: ' + ABudget.TimeStep;
  end;
  if ABudget.Zone = 'MOC3D' then
  begin
    result := result + '; Transport Step: ' + ABudget.TransportStep
       + '; Time: ' + ABudget.Time;
  end;
end;

procedure TfrmZoneBdgtReader.comboZoneChange(Sender: TObject);
var
  Index : integer;
  ABudget : TBudget;
  InnerIndex : Integer;
  ABudgetItem : TBudgetItem;
begin
  Screen.Cursor := crHourGlass;
  try
    comboTimeStep.Items.Clear;
    rzckIn.Items.Clear;
    rzckOut.Items.Clear;
    if comboZone.ItemIndex > -1 then
    begin
      for Index := 0 to BudgetList.Count - 1 do
      begin
        ABudget := BudgetList[Index];
        if comboZone.Text = ABudget.Zone then
        begin
          comboTimeStep.Items.Add(TimeText(ABudget));

          for InnerIndex := 0 to ABudget.Inlist.Count -1 do
          begin
            ABudgetItem := ABudget.Inlist[InnerIndex];
            if rzckIn.Items.IndexOf(ABudgetItem.Name) < 0 then
            begin
              rzckIn.Items.Add(ABudgetItem.Name);
            end;
          end;

          for InnerIndex := 0 to ABudget.Outlist.Count -1 do
          begin
            ABudgetItem := ABudget.Outlist[InnerIndex];
            if rzckOut.Items.IndexOf(ABudgetItem.Name) < 0 then
            begin
              rzckOut.Items.Add(ABudgetItem.Name);
            end;
          end;
        end;
      end;
      if comboTimeStep.Items.Count > 0 then
      begin
        comboTimeStep.ItemIndex := 0;
      end;
    end;
    cbInClick(Sender);
    cbOutClick(Sender);

    PlotResults;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TfrmZoneBdgtReader.BudgetListClear;
var
  Index : integer;
begin
  for Index := BudgetList.Count -1 downto 0 do
  begin
    TBudget(BudgetList[Index]).Free;
  end;
  BudgetList.Clear;

end;

procedure TfrmZoneBdgtReader.PlotResults;
var
  Index, BudgetIndex, BudgetItemIndex : integer;
  BarSeries : TBarSeries;
  ABudget : TBudget;
  ABudgetItem : TBudgetItem;
  ALineSeries : TLineSeries;
  AColor : TColor;
begin
  chartZONEBDGT.Title.Text.Clear;
  for Index := chartZONEBDGT.SeriesList.Count - 1 downto 0 do
  begin
    chartZONEBDGT.Series[Index].Free;
  end;
  chartZONEBDGT.RemoveAllSeries;
  case rgPlotType.ItemIndex of
    0: // bar chart
      begin
        chartZONEBDGT.BottomAxis.Title.Caption := '';
        chartZONEBDGT.BottomAxis.Labels := False;
        AColor := clTeeColor;
        BarSeries := TBarSeries.Create(self);
        BarSeries.ParentChart := chartZONEBDGT;
        BarSeries.Marks.Visible := False;

        ABudget := nil;
        for Index := 0 to BudgetList.Count -1 do
        begin
          ABudget := BudgetList[Index];
          if (ABudget.Zone = comboZone.Text)
            and (comboTimeStep.text = TimeText(ABudget)) then
          begin
            if Pos('Transport',comboTimeStep.text)> 0 then
            begin
              chartZONEBDGT.Title.Text.Add(ABudget.Zone + '; '
                + Copy(comboTimeStep.text,1,Pos('Transport',comboTimeStep.text)-1));
              chartZONEBDGT.Title.Text.Add(
                Copy(comboTimeStep.text,Pos('Transport',comboTimeStep.text),
                  Length(comboTimeStep.text)));
            end
            else
            if Pos('SUTRA',comboZone.text)> 0 then
            begin
              chartZONEBDGT.Title.Text.Add('Net Rate of Change in');
              if Pos('Mass',comboZone.text)> 0 then
              begin
                chartZONEBDGT.Title.Text.Add('Fluid Mass (Mass/s)')
              end
              else
              if Pos('Energy',comboZone.text)> 0 then
              begin
                chartZONEBDGT.Title.Text.Add('Energy (Energy/s)')
              end
              else
              begin
                chartZONEBDGT.Title.Text.Add('Transported Species (Mass/s)')
              end;
              chartZONEBDGT.Title.Text.Add(comboTimeStep.text);
            end
            else
            begin
              chartZONEBDGT.Title.Text.Add(ABudget.Zone + '; ' + comboTimeStep.text);
            end;
            break;
          end;
        end;
        if ABudget <> nil then
        begin
          for Index := 0 to rzckIn.Items.Count - 1 do
          begin
            if rzckIn.ItemChecked[Index] then
            begin
              ABudgetItem := ABudget.InList[Index];
              if Pos('SUTRA',ABudget.Zone) > 0  then
              begin
                BarSeries.Add(ABudgetItem.RealValue,ABudgetItem.Name,AColor);
              end
              else
              begin
                BarSeries.Add(ABudgetItem.RealValue,'In: ' + ABudgetItem.Name,AColor);
              end;
              AColor := GetColor(AColor);
            end;
          end;

          for Index := 0 to rzckOut.Items.Count - 1 do
          begin
            if rzckOut.ItemChecked[Index] then
            begin
              ABudgetItem := ABudget.OutList[Index];
              BarSeries.Add(ABudgetItem.RealValue,'Out: ' + ABudgetItem.Name,AColor);
              AColor := GetColor(AColor);
            end;
          end;

          if cbDecay.Checked then
          begin
            BarSeries.Add(ABudget.GetDecay, cbDecay.Caption,AColor);
            AColor := GetColor(AColor);
          end;

          if cbStoredMass.Checked then
          begin
            BarSeries.Add(ABudget.GetStoredMass, cbStoredMass.Caption,AColor);
            AColor := GetColor(AColor);
          end;

          if cbInOut.Checked then
          begin
            BarSeries.Add(ABudget.GetInMinusOut, cbInOut.Caption ,AColor);
            AColor := GetColor(AColor);
          end;

          if cbDiscrepancy.Checked then
          begin
            BarSeries.Add(ABudget.GetDiscrepancy, cbDiscrepancy.Caption ,AColor);
            // The following line will be needed if more options are added.
//            AColor := GetColor(AColor);
          end;

        end;

      end;
    1: // time series
      begin
//        AnotherColor := clAqua;
        AColor := clAqua;
        if Pos('SUTRA',comboZone.text)> 0 then
        begin
          chartZONEBDGT.Title.Text.Add('Time Series: Net Rate of Change in');
          if Pos('Mass',comboZone.text)> 0 then
          begin
            chartZONEBDGT.Title.Text.Add('Fluid Mass (Mass/s)')
          end
          else
          if Pos('Energy',comboZone.text)> 0 then
          begin
            chartZONEBDGT.Title.Text.Add('Energy (Energy/s)')
          end
          else
          begin
            chartZONEBDGT.Title.Text.Add('Transported Species (Mass/s)')
          end;
        end
        else
        begin
          chartZONEBDGT.Title.Text.Add('Time Series: ' + comboZone.Text);
        end;
        chartZONEBDGT.BottomAxis.Labels := True;
        chartZONEBDGT.BottomAxis.Title.Caption := 'Stored Time Step';
        for BudgetItemIndex := 0 to rzckIn.Items.Count -1 do
        begin
          if rzckIn.ItemChecked[BudgetItemIndex] then
          begin
            AColor := GetColor(AColor);
            ALineSeries := TLineSeries.Create(self);
            ALineSeries.LinePen.Color := AColor;
            ALineSeries.AreaColor := AColor;
            ALineSeries.AreaLinesPen.Color := AColor;
            ALineSeries.SeriesColor := AColor;
            ALineSeries.ParentChart := chartZONEBDGT;
            ALineSeries.Pointer.Visible := True;
            if rgDataSource.ItemIndex = 3 then
            begin  // SUTRA
              ALineSeries.Title := rzckIn.Items[BudgetItemIndex];
            end
            else
            begin
              ALineSeries.Title := 'In: ' + rzckIn.Items[BudgetItemIndex];
            end;
            for BudgetIndex := 0 to BudgetList.Count - 1 do
            begin
              ABudget := BudgetList[BudgetIndex];
              if (ABudget.Zone = comboZone.Text) then
              begin
                ABudgetItem := ABudget.InList[BudgetItemIndex];
                ALineSeries.AddY(ABudgetItem.RealValue,'',AColor);
              end;
            end;
          end;
        end;
        for BudgetItemIndex := 0 to rzckOut.Items.Count -1 do
        begin
          if rzckOut.ItemChecked[BudgetItemIndex] then
          begin
            AColor := GetColor(AColor);
            ALineSeries := TLineSeries.Create(self);
            ALineSeries.LinePen.Color := AColor;
            ALineSeries.AreaColor := AColor;
            ALineSeries.AreaLinesPen.Color := AColor;
            ALineSeries.SeriesColor := AColor;
            ALineSeries.ParentChart := chartZONEBDGT;
            ALineSeries.Pointer.Visible := True;
            ALineSeries.Title := 'Out: ' + rzckOut.Items[BudgetItemIndex];
            for BudgetIndex := 0 to BudgetList.Count - 1 do
            begin
              ABudget := BudgetList[BudgetIndex];
              if (ABudget.Zone = comboZone.Text) then
              begin
                ABudgetItem := ABudget.OutList[BudgetItemIndex];
                ALineSeries.AddY(ABudgetItem.RealValue,'',AColor);
              end;
            end;
          end;
        end;
        if cbDecay.Checked then
        begin
          AColor := GetColor(AColor);
          ALineSeries := TLineSeries.Create(self);
          ALineSeries.LinePen.Color := AColor;
          ALineSeries.AreaColor := AColor;
          ALineSeries.AreaLinesPen.Color := AColor;
          ALineSeries.SeriesColor := AColor;
          ALineSeries.ParentChart := chartZONEBDGT;
          ALineSeries.Pointer.Visible := True;
          ALineSeries.Title := cbDecay.Caption;
          for BudgetIndex := 0 to BudgetList.Count - 1 do
          begin
            ABudget := BudgetList[BudgetIndex];
            if (ABudget.Zone = comboZone.Text) then
            begin
              ALineSeries.AddY(ABudget.GetDecay,'',AColor);
            end;
          end;
        end;
        if cbStoredMass.Checked then
        begin
          AColor := GetColor(AColor);
          ALineSeries := TLineSeries.Create(self);
          ALineSeries.LinePen.Color := AColor;
          ALineSeries.AreaColor := AColor;
          ALineSeries.AreaLinesPen.Color := AColor;
          ALineSeries.SeriesColor := AColor;
          ALineSeries.ParentChart := chartZONEBDGT;
          ALineSeries.Pointer.Visible := True;
          ALineSeries.Title := cbStoredMass.Caption;
          for BudgetIndex := 0 to BudgetList.Count - 1 do
          begin
            ABudget := BudgetList[BudgetIndex];
            if (ABudget.Zone = comboZone.Text) then
            begin
              ALineSeries.AddY(ABudget.GetStoredMass,'',AColor);
            end;
          end;
        end;
        if cbInOut.Checked then
        begin
          AColor := GetColor(AColor);
          ALineSeries := TLineSeries.Create(self);
          ALineSeries.LinePen.Color := AColor;
          ALineSeries.AreaColor := AColor;
          ALineSeries.AreaLinesPen.Color := AColor;
          ALineSeries.SeriesColor := AColor;
          ALineSeries.ParentChart := chartZONEBDGT;
          ALineSeries.Pointer.Visible := True;
          ALineSeries.Title := cbInOut.Caption;
          for BudgetIndex := 0 to BudgetList.Count - 1 do
          begin
            ABudget := BudgetList[BudgetIndex];
            if (ABudget.Zone = comboZone.Text) then
            begin
              ALineSeries.AddY(ABudget.GetInMinusOut,'',AColor);
            end;
          end;
        end;
        if cbDiscrepancy.Checked then
        begin
          AColor := GetColor(AColor);
          ALineSeries := TLineSeries.Create(self);
          ALineSeries.LinePen.Color := AColor;
          ALineSeries.AreaColor := AColor;
          ALineSeries.AreaLinesPen.Color := AColor;
          ALineSeries.SeriesColor := AColor;
          ALineSeries.ParentChart := chartZONEBDGT;
          ALineSeries.Pointer.Visible := True;
          ALineSeries.Title := cbDiscrepancy.Caption;
          for BudgetIndex := 0 to BudgetList.Count - 1 do
          begin
            ABudget := BudgetList[BudgetIndex];
            if (ABudget.Zone = comboZone.Text) then
            begin
              ALineSeries.AddY(ABudget.GetDiscrepancy,'',AColor);
            end;
          end;
        end;
      end;
    else
      begin
        ShowMessage('No plot type selected');
      end;
  end;

end;

function TfrmZoneBdgtReader.GetColor(AColor : TColor) : TColor;
begin
  case AColor of
    clAqua     : Result := clBlack   ;
    clBlack    : Result := clBlue    ;
    clBlue     : Result := clDkGray  ;
    clDkGray   : Result := clFuchsia ;
    clFuchsia  : Result := clGreen    ;
    clGreen    : Result := clLime    ;
    clLime     : Result := clMaroon  ;
//    clLtGray   : Result := clMaroon  ;
    clMaroon   : Result := clNavy    ;
    clNavy     : Result := clOlive   ;
    clOlive    : Result := clPurple  ;
    clPurple   : Result := clRed     ;
    clRed      : Result := clTeal  ;
    clTeal     : Result := clWhite   ;
    clWhite    : Result := clYellow  ;
    clYellow   : Result := clAqua    ;
    else
      result := clAqua;
  end;

end;

procedure TfrmZoneBdgtReader.comboTimeStepChange(Sender: TObject);
begin
  PlotResults;
end;

procedure TfrmZoneBdgtReader.rzckChange(Sender: TObject; Index: Integer;
  NewState: TCheckBoxState);
begin
  PlotResults;

end;

procedure TfrmZoneBdgtReader.cbInOutClick(Sender: TObject);
begin
  PlotResults;

end;

procedure TfrmZoneBdgtReader.cbDiscrepancyClick(Sender: TObject);
begin
  PlotResults;

end;

procedure TfrmZoneBdgtReader.btnSaveClick(Sender: TObject);
var
  AStringList : TStringList;
  ZoneIndex : integer;
  AString : string;
  BudgetIndex, BudgetItemIndex : Integer;
  CurrentZone : string;
  ABudget : TBudget;
  ABudgetItem : TBudgetItem;
  TitleString : string;
  FileName : string;
  Directory : string;
begin
  FileName := OpenDialog1.FileName;
  Directory := ExtractFilePath(FileName);
  FileName :=  ExtractFileName(FileName);
  if Pos('.',FileName) > 0 then
  begin
    FileName := Copy(FileName,1,Pos('.',FileName)-1);
  end;
  SaveDialog1.FileName := Directory + FileName + '.' + SaveDialog1.DefaultExt;
  if SaveDialog1.Execute then
  begin
    AStringList := TStringList.Create;
    try
      for ZoneIndex := 0 to comboZone.Items.Count -1 do
      begin
        CurrentZone := comboZone.Items[ZoneIndex];
        AStringList.Add('');
        AStringList.Add(CurrentZone);
        if Pos('SUTRA',CurrentZone) > 0 then
        begin
          TitleString :=  'Time Step';
        end
        else
        begin
          TitleString := 'Stress Period' + Chr(9) + 'Time Step';
        end;
        if CurrentZone = 'MOC3D' then
        begin
          TitleString := TitleString + Chr(9) + 'Transport Step'
            + Chr(9) + 'Time';
        end;
        for BudgetIndex := 0 to BudgetList.Count - 1 do
        begin
          ABudget := BudgetList[BudgetIndex];
          if (ABudget.Zone = CurrentZone) then
          begin
            if Pos('SUTRA',CurrentZone) > 0 then
            begin
              AString := ABudget.TimeStep;
            end
            else
            begin
              AString := ABudget.StressPeriod + chr(9) + ABudget.TimeStep;
            end;
            if CurrentZone = 'MOC3D' then
            begin
              AString := AString + Chr(9) + ABudget.TransportStep
                 + Chr(9) + ABudget.Time;
            end;

            For BudgetItemIndex := 0 to ABudget.InList.Count -1 do
            begin
              ABudgetItem := ABudget.InList[BudgetItemIndex];
              AString := AString + Chr(9) + ABudgetItem.value;
            end;

            if BudgetIndex div comboZone.Items.Count = 0 then
            begin
              For BudgetItemIndex := 0 to ABudget.InList.Count -1 do
              begin
                ABudgetItem := ABudget.InList[BudgetItemIndex];
                TitleString := TitleString + Chr(9) + 'In: ' + ABudgetItem.Name;
              end;
            end;

            For BudgetItemIndex := 0 to ABudget.OutList.Count -1 do
            begin
              ABudgetItem := ABudget.OutList[BudgetItemIndex];
              AString := AString + Chr(9) + ABudgetItem.value;
            end;

            if BudgetIndex div comboZone.Items.Count = 0 then
            begin
              For BudgetItemIndex := 0 to ABudget.OutList.Count -1 do
              begin
                ABudgetItem := ABudget.OutList[BudgetItemIndex];
                TitleString := TitleString + Chr(9) + 'Out: ' + ABudgetItem.Name;
              end;
            end;

            if CurrentZone = 'MOC3D' then
            begin
              AString := AString + Chr(9) + ABudget.Decay
                 + Chr(9) + ABudget.StoredMass;
            end;

            if Pos('SUTRA',CurrentZone) > 0 then
            begin
              AString := AString + Chr(9) + ABudget.InMinusOut
                + Chr(9) + ABudget.Discrepancy;
            end;

            if BudgetIndex div comboZone.Items.Count = 0 then
            begin
              if CurrentZone = 'MOC3D' then
              begin
                TitleString := TitleString + Chr(9) + cbDecay.Caption
                   + Chr(9) + cbStoredMass.Caption;
              end;
              if Pos('SUTRA',CurrentZone) > 0 then
              begin
                TitleString := TitleString + Chr(9) + cbInOut.Caption
                   + Chr(9) + cbDiscrepancy.Caption;
              end;
              AStringList.Add(TitleString);
            end;
          end;

          if (ABudget.Zone = CurrentZone) then
          begin
            AStringList.Add(AString);
          end;
        end;
        AStringList.SaveToFile(SaveDialog1.FileName);
      end;
    finally
      AStringList.Free;
    end;
  end;
end;

procedure TfrmZoneBdgtReader.About1Click(Sender: TObject);
begin
  frmAbout.Show;
end;

procedure TfrmZoneBdgtReader.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TfrmZoneBdgtReader.cbOutClick(Sender: TObject);
var
  Index: integer;
begin
  for Index := 0 to rzckOut.Items.Count -1 do
  begin
    rzckOut.ItemChecked[Index] := cbOut.Checked;
  end;
  PlotResults;
end;

procedure TfrmZoneBdgtReader.cbInClick(Sender: TObject);
var
  Index: integer;
begin
  for Index := 0 to rzckIn.Items.Count -1 do
  begin
    rzckIn.ItemChecked[Index] := cbIn.Checked;
  end;
  PlotResults;
end;

procedure TfrmZoneBdgtReader.rgDataSourceClick(Sender: TObject);
begin
  Case rgDataSource.ItemIndex of
    0: // ZONEBDGT
      begin
          OpenDialog1.Filter := 'Zonebudget Listing files (*.zbl)|*.zbl|All Files (*.*)|*.*';
          cbDecay.Enabled := False;
          cbDecay.Checked := False;
          cbStoredMass.Enabled := False;
          cbStoredMass.Checked := False;
          cbInOut.Enabled := True;
          cbInOut.Checked := True;
          cbDiscrepancy.Enabled := True;
          cbDiscrepancy.Checked := True;
          cbInOut.Caption := 'IN - OUT';
          cbOut.Visible := True;
          Panel8.Visible := True;
          cbIn.Caption := '"In" Budget Items';
          chartZONEBDGT.LeftAxis.Title.Caption := '';
          btnOpen.Hint := 'Open ZONEBDGT file';
          btnSave.Hint := 'Save ZONEBDGT data in tab-delimited format';
      end;
    1: // MODFLOW
      begin
          OpenDialog1.Filter := 'MODFLOW Listing files |*.lst;*.out;output.dat|All Files (*.*)|*.*';
          cbDecay.Enabled := False;
          cbDecay.Checked := False;
          cbStoredMass.Enabled := False;
          cbStoredMass.Checked := False;
          cbInOut.Enabled := True;
          cbInOut.Checked := True;
          cbDiscrepancy.Enabled := True;
          cbDiscrepancy.Checked := True;
          cbInOut.Caption := 'IN - OUT';
          cbOut.Visible := True;
          Panel8.Visible := True;
          cbIn.Caption := '"In" Budget Items';
          chartZONEBDGT.LeftAxis.Title.Caption := '';
          btnOpen.Hint := 'Open MODFLOW file';
          btnSave.Hint := 'Save MODFLOW data in tab-delimited format';
      end;
    2: // MOC3D
      begin
          OpenDialog1.Filter := 'MOC3D Listing files|*.out;Moc3d.lst|All Files (*.*)|*.*';
          cbDecay.Enabled := True;
          cbDecay.Checked := True;
          cbStoredMass.Enabled := True;
          cbStoredMass.Checked := True;
          cbInOut.Enabled := True;
          cbInOut.Checked := True;
          cbDiscrepancy.Enabled := True;
          cbDiscrepancy.Checked := True;
          cbInOut.Caption := 'Residual';
          cbOut.Visible := True;
          Panel8.Visible := True;
          cbIn.Caption := '"In" Budget Items';
          chartZONEBDGT.LeftAxis.Title.Caption := '';
          btnOpen.Hint := 'Open MOC3D file';
          btnSave.Hint := 'Save MOC3D data in tab-delimited format';
      end;
    3: // SUTRA
      begin
//          OpenDialog1.Filter := 'SUTRA Listing files (*.d6)|*.d6|All Files (*.*)|*.*';
          OpenDialog1.Filter := 'SUTRA Listing files (*.lst)|*.lst|All Files (*.*)|*.*';
          cbDecay.Enabled := False;
          cbDecay.Checked := False;
          cbStoredMass.Enabled := False;
          cbStoredMass.Checked := False;
          cbInOut.Enabled := False;
          cbInOut.Checked := False;
          cbDiscrepancy.Enabled := False;
          cbDiscrepancy.Checked := False;
          cbOut.Visible := False;
          cbOut.Checked := False;
          Panel8.Visible := False;
          cbIn.Caption := 'Budget Items';
          chartZONEBDGT.LeftAxis.Title.Caption := 'Decrease(-) / Increase(+)';
          btnOpen.Hint := 'Open SUTRA file';
          btnSave.Hint := 'Save SUTRA data in tab-delimited format';
      end;
    4: // MT3D
      begin
          OpenDialog1.Filter := 'MT3D Listing files|*.mls;*.out;output.mt3|All Files (*.*)|*.*';
          cbDecay.Enabled := False;
          cbDecay.Checked := False;
          cbStoredMass.Enabled := False;
          cbStoredMass.Checked := False;
          cbInOut.Enabled := True;
          cbInOut.Checked := True;
          cbDiscrepancy.Enabled := True;
          cbDiscrepancy.Checked := True;
          cbInOut.Caption := 'NET (IN - OUT)';
          cbOut.Visible := True;
          Panel8.Visible := True;
          cbIn.Caption := '"In" Budget Items';
          chartZONEBDGT.LeftAxis.Title.Caption := '';
          btnOpen.Hint := 'Open MT3D file';
          btnSave.Hint := 'Save MT3D data in tab-delimited format';
      end;
  end;
  FormResize(Sender);
end;

procedure TfrmZoneBdgtReader.cbDecayClick(Sender: TObject);
begin
  PlotResults;
end;

procedure TfrmZoneBdgtReader.cbStoredMassClick(Sender: TObject);
begin
  PlotResults;
end;

procedure TfrmZoneBdgtReader.FormResize(Sender: TObject);
begin
  Panel8.Width := (Panel8.Width + Panel6.Width) div 2;

end;

procedure SetDecimalSign;
var
  CommaSeparator : array[0..255] of Char;
begin
  GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SDECIMAL,@CommaSeparator,255);
  DecimalSign := CommaSeparator[0];
end;

procedure TfrmZoneBdgtReader.sbImageClick(Sender: TObject);
var
  OldFilter : string;
  OldDefaultExtension : string;
  OldFilterIndex : integer;
begin
  OldFilter := SaveDialog1.Filter;
  OldDefaultExtension := SaveDialog1.DefaultExt;
  OldFilterIndex := SaveDialog1.FilterIndex;
  try
    SaveDialog1.Filter := 'Bitmap (*.bmp)|*.bmp|Enhanced Windows Metafile (*.emf)|*.emf|Windows Metafile (*.wmf)|*.wmf';
    SaveDialog1.DefaultExt := 'wmf';
    SaveDialog1.FilterIndex := 3;
    if SaveDialog1.Execute then
    begin
      if LowerCase(ExtractFileExt(SaveDialog1.FileName)) = '.bmp' then
      begin
        chartZONEBDGT.SaveToBitmapFile(SaveDialog1.FileName);
      end
      else if LowerCase(ExtractFileExt(SaveDialog1.FileName)) = '.emf' then
      begin
        chartZONEBDGT.SaveToMetafileEnh(SaveDialog1.FileName);
      end
      else if LowerCase(ExtractFileExt(SaveDialog1.FileName)) = '.wmf' then
      begin
        chartZONEBDGT.SaveToMetafile(SaveDialog1.FileName);
      end;
    end;
  finally
    SaveDialog1.Filter := OldFilter;
    SaveDialog1.DefaultExt := OldDefaultExtension;
    SaveDialog1.FilterIndex := OldFilterIndex;
  end;

end;

procedure TfrmZoneBdgtReader.sbFormatClick(Sender: TObject);
begin
  frmFormat.GetData(chartZONEBDGT);
  frmFormat.ShowModal;
end;

procedure TfrmZoneBdgtReader.ToolBar1MouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  Screen.Cursor := crDefault;
end;

Initialization
SetDecimalSign;

end.
