unit frmFormatUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ArgusDataEntry, Chart;

type
  TfrmFormat = class(TForm)
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    memoTitle: TMemo;
    Label2: TLabel;
    GroupBox1: TGroupBox;
    Label3: TLabel;
    edXAxisTitle: TEdit;
    cbAutoX: TCheckBox;
    adeXMin: TArgusDataEntry;
    Label1: TLabel;
    adeXMax: TArgusDataEntry;
    lblXMax: TLabel;
    adeXTickInterval: TArgusDataEntry;
    Label6: TLabel;
    GroupBox2: TGroupBox;
    Label5: TLabel;
    edYAxisTitle: TEdit;
    cbAutoY: TCheckBox;
    adeYMin: TArgusDataEntry;
    adeYMax: TArgusDataEntry;
    adeYTickInterval: TArgusDataEntry;
    lblYMin: TLabel;
    Label4: TLabel;
    Label7: TLabel;
    cbXGrid: TCheckBox;
    cbYGrid: TCheckBox;
    FontDialog1: TFontDialog;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    Button5: TButton;
    cbLegend: TCheckBox;
    btnLegendFont: TButton;
    procedure cbAutoXClick(Sender: TObject);
    procedure cbAutoYClick(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure btnLegendFontClick(Sender: TObject);
  private
    { Private declarations }
    Chart : TChart;
    XTickFont, YTickFont, LegendFont : TFont;
    procedure SetData;
    function PickFont(AFont: TFont): TFont;
  public
    procedure GetData(AChart : TChart);
    { Public declarations }
  end;

var
  frmFormat: TfrmFormat;

implementation

{$R *.DFM}

procedure TfrmFormat.cbAutoXClick(Sender: TObject);
begin
  adeXMin.Enabled := not cbAutoX.Checked;
  adeXMax.Enabled := not cbAutoX.Checked;
end;

procedure TfrmFormat.cbAutoYClick(Sender: TObject);
begin
  adeYMin.Enabled := not cbAutoY.Checked;
  adeYMax.Enabled := not cbAutoY.Checked;

end;

procedure TfrmFormat.BitBtn2Click(Sender: TObject);
begin
  SetData;
end;

procedure TfrmFormat.GetData(AChart : TChart);
begin
  Chart := AChart;
  with AChart do
  begin
    memoTitle.Lines.Assign(Title.Text);
    edXAxisTitle.Text := BottomAxis.Title.Caption;
    edYAxisTitle.Text := LeftAxis.Title.Caption;

    memoTitle.Font := Title.Font;
    edXAxisTitle.Font := BottomAxis.Title.Font;
    edYAxisTitle.Font := LeftAxis.Title.Font;
    XTickFont.Assign(BottomAxis.LabelsFont);
    YTickFont.Assign(LeftAxis.LabelsFont);
    LegendFont.Assign(Legend.Font);

    cbAutoX.Checked := BottomAxis.Automatic;
    adeXMin.Text := FloatToStr(BottomAxis.Minimum);
    adeXMax.Text := FloatToStr(BottomAxis.Maximum);
    cbAutoY.Checked := LeftAxis.Automatic;
    adeYMin.Text := FloatToStr(LeftAxis.Minimum);
    adeYMax.Text := FloatToStr(LeftAxis.Maximum);

    adeXTickInterval.Text := FloatToStr(BottomAxis.Increment);
    adeYTickInterval.Text := FloatToStr(LeftAxis.Increment);

    cbXGrid.Checked := BottomAxis.Grid.Visible;
    cbYGrid.Checked := LeftAxis.Grid.Visible;

    cbLegend.Checked := Legend.Visible;
    cbLegend.Enabled := Legend.TotalLegendItems > 0;
    btnLegendFont.Enabled := cbLegend.Enabled;

    cbAutoXClick(cbAutoX);
    cbAutoYClick(cbAutoY);
  end;
end;

procedure TfrmFormat.SetData;
begin
  with Chart do
  begin
    BottomAxis.Automatic            := cbAutoX.Checked ;
    BottomAxis.Minimum  := StrToFloat(adeXMin.Text);
    BottomAxis.Maximum  := StrToFloat(adeXMax.Text);
    LeftAxis.Automatic              := cbAutoY.Checked;
    LeftAxis.Minimum    := StrToFloat(adeYMin.Text);
    LeftAxis.Maximum    := StrToFloat(adeYMax.Text);

    Title.Text.Assign(memoTitle.Lines);
    BottomAxis.Title.Caption := edXAxisTitle.Text;
    LeftAxis.Title.Caption   := edYAxisTitle.Text;

    BottomAxis.Increment := StrToFloat(adeXTickInterval.Text);
    LeftAxis.Increment   := StrToFloat(adeYTickInterval.Text);

    BottomAxis.Grid.Visible := cbXGrid.Checked;
    LeftAxis.Grid.Visible   := cbYGrid.Checked;

    Title.Font := memoTitle.Font;
    BottomAxis.Title.Font := edXAxisTitle.Font;
    LeftAxis.Title.Font := edYAxisTitle.Font;
    BottomAxis.LabelsFont := XTickFont;
    LeftAxis.LabelsFont := YTickFont;
    Legend.Font := LegendFont;

    Legend.Visible := cbLegend.Checked;
  end;
end;


procedure TfrmFormat.FormCreate(Sender: TObject);
begin
  XTickFont := TFont.Create;
  YTickFont := TFont.Create;
  LegendFont := TFont.Create;
end;

procedure TfrmFormat.FormDestroy(Sender: TObject);
begin
  XTickFont.Free;
  YTickFont.Free;
  LegendFont.Free;
end;

function TfrmFormat.PickFont(AFont : TFont) : TFont;
begin
  FontDialog1.Font := AFont;
  if FontDialog1.Execute then
  begin
    result := FontDialog1.Font
  end
  else
  begin
    result := AFont;
  end;
end;

procedure TfrmFormat.Button1Click(Sender: TObject);
begin
  memoTitle.Font.Assign(PickFont(memoTitle.Font));
end;

procedure TfrmFormat.Button2Click(Sender: TObject);
begin
  edXAxisTitle.Font.Assign(PickFont(edXAxisTitle.Font));
end;

procedure TfrmFormat.Button5Click(Sender: TObject);
begin
  edYAxisTitle.Font.Assign(PickFont(edYAxisTitle.Font));
end;

procedure TfrmFormat.Button3Click(Sender: TObject);
begin
  XTickFont.Assign(PickFont(XTickFont));
end;

procedure TfrmFormat.Button4Click(Sender: TObject);
begin
  YTickFont.Assign(PickFont(YTickFont));
end;

procedure TfrmFormat.btnLegendFontClick(Sender: TObject);
begin
  LegendFont.Assign(PickFont(LegendFont));
end;

end.
